#! /bin/csh -f
#
#
#	PattEQ:  calculate Patterson equivalence 
#
#	given:
#		two pdb files
#		a space group
#
#
# make sure nawk works
alias nawk /usr/bin/nawk
nawk 'BEGIN{exit}' >& /dev/null
if($status) alias nawk awk
#
set tempfile = patteq_temp
set logfile  = patteq.log
set file1 = ""
set file2 = ""

# defaults
set CELL     = "100 100 100 90 90 90"
set SG       = "P1"


goto Setup
# scan the command line for "site" files
ReturnFromSetup:

if(($files != 2)||("$SG" == "")) then
    if("$SG" == "") set $SG = P212121
    cat << EOF
usage: $0 $SG file1.pdb file2.pdb

where:
$SG	   - is your space group
filex.pdb  - can be: a pdb file, 
                     a shelx "res" file, 
                     a solve.status file,
		     an mlphare script, 
                     a random "site" file containing fractional x, y, z

EOF
    rm -f ${tempfile}xyz* >& /dev/null
    rm -f ${tempfile}cell* >& /dev/null
    exit 9
endif

# make sure we have cell info?
set cells = `nawk 'NF==6' ${tempfile}cell[12] | wc -l`
if("$cells" == 0) then
    echo "no unit cell given! Assuming: $CELL"
else
    # use the first cell (why not? )
    set CELL = `nawk 'NF==6' ${tempfile}cell[12] | head -1`
    echo "Unit Cell:   $CELL"
endif
echo "Space Group: $SG"
echo ""


# convert any fractional-coordinate files to Angstrom coordinates
foreach file ( 1 2 )
    grep -i "frac" ${tempfile}xyz$file >& /dev/null
    if($status) then
	# this was already an Angstrom-coordinate file
	nawk 'NF>3' ${tempfile}xyz$file >! ${tempfile}
	mv ${tempfile} ${tempfile}xyz$file
    else
	# this was a fractional-coordinate file, convert it to Angstrom
	cat ${tempfile}xyz$file |\
	nawk 'NF>3{printf "%5d%10.5f%10.5f%10.5f%10.5f%5.2f%5d%10d%2s%-3d%3s %1s\n", \
	       1,  $1, $2, $3, $5, $4,  "38", 1, "HA", 1, "IUM", " "}' |\
	cat >! ${tempfile}.frac

	# use coordconv to convert fractional MLphare coordinates to Angstroms
	coordconv XYZIN ${tempfile}.frac \
	         XYZOUT ${tempfile}.pdb << EOF-conv >> $logfile
	CELL $CELL
	INPUT FRAC
	OUTPUT PDB ORTH 1
	END
EOF-conv
	rm -f ${tempfile}.frac >& /dev/null
	
	# extract the Angstrom coordinates
	cat "${tempfile}.pdb" |\
	nawk '/^ATOM/{X=substr($0, 31, 8); Y=substr($0, 39, 8); Z=substr($0, 47, 8);\
		      Occ=substr($0, 55, 6); B=substr($0, 61, 6); \
		      print X, Y, Z, Occ, B, "orth" }' |\
	cat >! ${tempfile}xyz$file
	
	rm -f ${tempfile}.pdb >& /dev/null
    endif
end

########################
# at this point, everything is in orthogonal coordinates
########################

# figure out whether or not to permute axes (based on cell equivalence)
cat ${tempfile}cell1 ${tempfile}cell2 |\
nawk 'NR==1{A=$1;B=$2;C=$3} \
      NR==2{x=$1;y=$2;z=$3;\
      print (A-x)^2+(B-y)^2+(C-z)^2, "XYZ";\
      print (A-y)^2+(B-z)^2+(C-x)^2, "YZX";\
      print (A-z)^2+(B-x)^2+(C-y)^2, "ZXY";}' |\
sort -n | head -1 >! ${tempfile}permutation
set permutation = `nawk '{print $NF}' ${tempfile}permutation`
rm -f ${tempfile}permutation >& /dev/null

if("$permutation" == "YZX") then
    # apply this to SECOND file
    echo "permuting $file2"
    echo "by new XYZ = old $permutation"
    cat ${tempfile}xyz2 |\
    nawk 'NF==3{print $2, $3, $1}' |\
    cat >! ${tempfile}
    mv ${tempfile} ${tempfile}xyz2 
endif
if("$permutation" == "ZXY") then
    # apply this to SECOND file
    echo "permuting $file2"
    echo "by new XYZ = old $permutation"
    cat ${tempfile}xyz2 |\
    nawk 'NF==3{print $3, $1, $2}' |\
    cat >! ${tempfile}
    mv ${tempfile} ${tempfile}xyz2 
endif

########################
# generate a "fake" Patterson map for each

# first, decide on a resolution limit from the B-factors in the files
set hiRES = `nawk '{print 3*sqrt($5/79)}' ${tempfile}xyz[12] | sort -n | head -1`
set mapRES = "$hiRES"
set GRID
set XYZLIM

# handle input maps
foreach file ( 1 2 )
    grep "GRID" ${tempfile}xyz$file >& /dev/null
    if(! $status) then
	# this was a map file
	if("$GRID" == "") then
	    set GRID   = `grep "GRID" ${tempfile}xyz$file`
	    set XYZLIM = `grep "XYZLIM" ${tempfile}xyz$file`
	    set mapRES = `echo "$GRID $CELL" | nawk '{print 3*$5/$2; print 3*$6/$3; print 3*$7/$4}' | sort -n | tail -1`
	    
	    # this file is on the correct grid
	    if($file == 1) cp $file1 ${tempfile}patt${file}.map
	    if($file == 2) cp $file2 ${tempfile}patt${file}.map
	    
	    continue
	endif
	
	# this map might not be on first map's grid
	set GRID2 = `grep "GRID" ${tempfile}xyz$file`
	set XYZLIM2 = `grep "XYZLIM" ${tempfile}xyz$file`
	if(("$GRID2" != "$GRID")||("$XYZLIM2" != "$XYZLIM")) then
	    # need to re-sample this map
	    echo "re-sampling $file2 to same grid as $file1"
	    goto resample
	endif
    endif
end
resampled:

# now turn site constellations into Pattersons
foreach file ( 1 2 )
    grep "GRID" ${tempfile}xyz$file >& /dev/null
    if(! $status) then
	# this was a map file (patt already taken care of)
	continue
    endif
        
    cat ${tempfile}xyz$file |\
    nawk '{++i;X=$1;Y=$2;Z=$3;Occ=$4;B=$5;\
    printf("ATOM  %5d  HA  IUM  %4d    %8.3f%8.3f%8.3f %5.2f%6.2f\n",i,i,X,Y,Z,Occ,B);}' |\
    cat >! ${tempfile}.pdb

    # back-transform the atoms to HKLs
    sfall xyzin ${tempfile}.pdb HKLOUT ${tempfile}.mtz << EOF >> $logfile
    MODE SFCALC XYZIN
    SYMM $SG
    CELL $CELL
    RESOLUTION 1000 $hiRES
EOF

    # now make the Patterson map
    fft hklin ${tempfile}.mtz mapout ${tempfile}patt${file}.map << EOF >> $logfile
    PATTERSON
    RESOLUTION 1000 $mapRES
    $GRID
    $XYZLIM
    LABIN F1=FC
EOF

    rm -f ${tempfile}.pdb >& /dev/null
    rm -f ${tempfile}.mtz >& /dev/null
end

echo "correlate section" |\
overlapmap mapin1 ${tempfile}patt1.map  mapin2 ${tempfile}patt2.map |\
nawk '/Total corr/{print $NF}'


set file = 2
cp ${tempfile}xyz$file ${tempfile}better
while($?NIX_BAD_SITES)
    
    # clear the score file
    echo -n >! ${tempfile}cc
    
    # try removing each site, in turn
    foreach site ( 0 `nawk '{print NR}' ${tempfile}better` )
	
	# print out what we are doing
	echo -n "$site  " | tee -a ${tempfile}cc
	
	cat ${tempfile}better |\
	nawk -v site=$site '{++i}\
	        NR!=site{X=$1;Y=$2;Z=$3;Occ=$4;B=$5;\
	printf("ATOM  %5d  HA  IUM  %4d    %8.3f%8.3f%8.3f %5.2f%6.2f\n",i,i,X,Y,Z,Occ,B);}' |\
	cat >! ${tempfile}.pdb
	
	# back-transform the atoms to HKLs
	sfall xyzin ${tempfile}.pdb HKLOUT ${tempfile}.mtz << EOF >> $logfile
	MODE SFCALC XYZIN
	SYMM $SG
	CELL $CELL
	RESOLUTION 1000 $hiRES
EOF
	
	# now make the "fake" Patterson map
	fft hklin ${tempfile}.mtz mapout ${tempfile}patt.map << EOF >> $logfile
	PATTERSON 
	LABIN F1=FC
EOF

	echo "correlate section" |\
	overlapmap mapin1 ${tempfile}patt1.map  mapin2 ${tempfile}patt.map |\
	nawk '/Total corr/{print $NF}' | tee -a ${tempfile}cc
    end
    
    # find the "worst" site
    # (the one that gives the best CC when removed)
    set badsite = `sort -nr +1 ${tempfile}cc | nawk '{print $1; exit}'`
    echo "Throwing out site # $badsite"
    
    # quit when removing any site reduces correlation
    if("$badsite" == 0) then
	# we are done throwing out sites
	echo "all sites seem okay"
	unset NIX_BAD_SITES	
    endif
    
    # remove this site
    cat ${tempfile}better |\
    nawk -v badsite=$badsite 'NR!=badsite' |\
    cat >! ${tempfile}
    mv ${tempfile} ${tempfile}better >& /dev/null
    
    # remove any sites that reduced CC
    cat ${tempfile}cc ${tempfile}better |\
    nawk 'NF==2{score[$1]=$2}\
          NF>2{++i;if(score[i]<score[0])print}' |\
    cat >! ${tempfile}
    
end

rm -f ${tempfile}.pdb >& /dev/null
rm -f ${tempfile}.mtz >& /dev/null

echo "GOTHERE"
exit


nawk '/PLAN/,/END/' SHELX/fh.res |\
nawk 'NF>6{print $3, $4, $5, 1, 80}' |\
nawk '{++i; print "ATOM H", i, $4, $1, $2, $3, $5}' |\
cat >! ${tempfile}atoms
vecref mapin ${tempfile}patt1.map ATOUT ${tempfile}atoms.pdb << EOF
SPACEGROUP P212121
RESOLUTION 2
CYCLES 20 0 0
@${tempfile}atoms
EOF
cat ${tempfile}atoms.pdb |\
nawk '{print $5, $6, $7, $4, $8}' |\
cat >! ${tempfile}frac


# now "score" one set of sites against the other Patterson



# generate a ridiculously small Patterson map
echo "1 1 1 1" >! ${tempfile}.hkl
f2mtz hklin ${tempfile}.hkl hklout ${tempfile}.mtz << EOF >> $logfile
CELL $CELL
SYMM $SG
LABO H K L F 
CTYP H H H F  
EOF
fft hklin ${tempfile}.mtz mapout ${tempfile}.map << EOF >> $logfile
PATTERSON 
LABIN F1=F
EOF

foreach file ( 1 2 )

    # use "vectors" to make Patterson vectors
    echo "SYMM $SG" |\
    cat - ${tempfile}xyz$file |\
    nawk 'NF==3{++i; printf "ATOM %c %s %s %s\n", i+48,$1,$2,$3 } /^SYMM/' |\
    vectors MAPIN ${tempfile}.map XYZOUT ${tempfile}.pdb | tee -a $logfile |\
    nawk '/Orthogonal coordinate/,/termination/{if(NF>11) print}' |\
    nawk '{print $5, $6, $7, substr($11,2,1), substr($11,3,1), $NF}' |\
    sort -n +5 >! ${tempfile}vectors
    rm -f ${tempfile}.pdb
    
    # build a translation table from ASCII characters to numbers
    cat ${tempfile}xyz$file |\
    nawk 'NF==3{++i; printf "label %c %d\n", i+48,i }' |\
    cat >! ${tempfile}labels
    
    # convert ascii atom labels to atom numbers
    cat ${tempfile}labels ${tempfile}vectors |\
    nawk -v file=$file '/^label/{label[$2]=$NF}\
        ! /^label/{print file, $1, $2, $3, label[$4], label[$5], $6}' |\
    cat >! ${tempfile}vectors$file
end
# clean up
rm -f ${tempfile}labels ${tempfile}vectors.pdb 

#####################
# ${tempfile}vectors$file has format:
# file# U V W atom1 atom2 length
#####################


# now we need to see if these Patterson vector sets are equivalent
echo "calculating vector differences..."
cat ${tempfile}vectors1 ${tempfile}vectors2 |\
nawk '$1==1{++n; U[n]=$2;V[n]=$3;W[n]=$4;a1[n]=$5;a2[n]=$6;L[n]=$7;}\
      $1==2{minDsqr=9999999999999; for(i=1;i<=n;++i){\
	      # use radius check to speed things up \
              if((($7-L[i])*($7-L[i]) < minDsqr)||(1)){\
		# calculate real distance \
		Dsqr = ($2-U[i])^2 + ($3-V[i])^2 + ($4-W[i])^2;\
	        if(Dsqr<minDsqr){minDsqr=Dsqr; mini=i}}};\
	    print sqrt(minDsqr), U[mini], V[mini], W[mini], a1[mini], a2[mini], $2, $3, $4, $5, $6 }' |\
sort -n >! ${tempfile}vector_diffs

#####################
# ${tempfile}vectors_diffs has format:
# minDist U1 V1 W1 atom1a atom1b U2 V2 W2 atom2a atom2b
#####################

# print out vector-vector distances
echo "  set1    set2  distance"
sort -n ${tempfile}vector_diffs |\
nawk '{printf "%3d-%-3d %3d-%-3d %f\n", $5, $6, $10, $11, $1}'

# calculate minimum atom score?
echo "set atom best delta-UVW"
nawk '{print $1, $5, $6, $10, $11}' ${tempfile}vector_diffs |\
sort -nr |\
nawk '{score1[$2]=$1;score1[$3]=$1; \
       score2[$4]=$1;score2[$5]=$1; \
       vector[$1]= $2 "-" $3 " " $4 "-" $5} \
   END{for(atom in score1){printf "1 %6d %f\n", atom, score1[atom]}\
       for(atom in score2){printf "2 %6d %f\n", atom, score2[atom]} }' |\
sort +0n -1 +2n -3

exit

# calculate mean atom score? 
cat ${tempfile}vector_diffs |\
nawk '{score1[$5]+=$1; score1[$6]+=$1;\
         ++n1[$5];       ++n1[$6];\
       score2[$10]+=$1;score2[$11]+=$1;\
         ++n2[$10];      ++n2[$11]} \
   END{for(atom in n1){printf "1 %3d %f\n", atom, score1[atom]/n1[atom]}\
       for(atom in n2){printf "2 %3d %f\n", atom, score2[atom]/n2[atom]} }' |\
sort +0n -1 +2n -3


Setup:
#########################################################################

#########################################################################

if(! $?CLIBD) then
    echo "set up CCP4 first! "
    exit
endif

# scan the command line for "site" files
set files = 0
foreach arg ( $* $file1 $file2 )
    # identify space groups
    if("$arg" =~ [pPcCIiFfRrHh][1-6]*) then
	set temp = `echo $arg | nawk '{print toupper($1)}'`
	if( -e $CLIBD/symop.lib) then
	    # maybe limit this to cannonical SGs only?
	    set temp = `nawk -v SG=$temp '$4 == toupper(SG) {print $4;exit}' $CLIBD/symop.lib | head -1`
	endif
	if("$temp" =~ [PpCcIiFfRr][1-6]*) then
	    # this was a real space group
	    set SG = "$temp"
	    continue
	else
	    echo "WARNING: $arg is not a valid space group! "
	endif
    endif
    
    # recognize pdb files (convert later)
    if(("$arg" =~ *.pdb)||("$arg" =~ *.brk)) then
	if(! -e "$arg") then
	    echo "WARNING: $arg does not exist! "
	    continue
	endif
	@ files = ( $files + 1 )
	
	# retrieve the unit cell
	cat $arg |\
	nawk '$1 ~ /^CRYS/{print $2,$3,$4,$5,$6,$7;exit}' |\
	cat >! ${tempfile}cell$files
	
	# try to reconstruct the space group?
	
	# sites are already in orthogonal coordinates (just hope we know what cell is! )
	cat "$arg" |\
	nawk '/^ATOM/{X=substr($0, 31, 8); Y=substr($0, 39, 8); Z=substr($0, 47, 8);\
		      Occ=substr($0, 55, 6); B=substr($0, 61, 6); \
		      print X, Y, Z, Occ, B, "orth" }' |\
	cat >! ${tempfile}xyz$files
	
	# remember the filenames
	if("$file1" == "") then
	    set file1 = "$arg"
	else
	    set file2 = "$arg"
	endif
	continue
    endif
    
    # recognize map files
    if(("$arg" =~ *.map)||("$arg" =~ *.ext)) then
	if(! -e "$arg") then
	    echo "WARNING: $arg does not exist! "
	    continue
	endif
	
	# verify the map
	echo "go" | mapdump mapin "$arg" >! ${tempfile}mapdump
	if($status) then
	    echo "WARNING: $arg is unreadable! skipping it."
	    rm -f ${tempfile}mapdump >& /dev/null
	    continue
	endif
	@ files = ( $files + 1 )
	
	# record the cell
	cat ${tempfile}mapdump |\
	nawk '/Cell dimensions/{print $4,$5,$6,$7,$8,$9;exit}' |\
	cat >! ${tempfile}cell$files
	
	# put other interesting info in "xyz" file
	cat ${tempfile}mapdump |\
	nawk '/Space-group/{print "SYMM", $NF}\
	  /Cell dimensions/{print "CELL",$4,$5,$6,$7,$8,$9}\
	    /Grid sampling/{print "GRID", $(NF-2), $(NF-1), $NF}\
    /Start and stop points/{limits["f"]=$(NF-5) " " $(NF-4); \
	                    limits["m"]=$(NF-3) " " $(NF-2); \
			    limits["s"]=$(NF-1) " " $NF}\
       /Fast, medium, slow/ && $NF ~ /[XYZ]/{print "AXIS", $(NF-2), $(NF-1), $NF;\
	    fms[$(NF-2)]="f";fms[$(NF-1)]="m";fms[$NF]="s"; \
	    print "XYZLIM", limits[fms["X"]], limits[fms["Y"]], limits[fms["Z"]];}' |\
	cat >! ${tempfile}xyz$files
	
	rm -f ${tempfile}mapdump >& /dev/null
	
	# list maps like any other kind of input "file"
	if("$file1" == "") then
	    set file1 = "$arg"
	else
	    set file2 = "$arg"
	endif
	continue
    endif
    
    # recognize and convert SHELX result file
    if("$arg" =~ *.res) then
	if(! -e "$arg") then
	    echo "WARNING: $arg does not exist! "
	    continue
	endif
	@ files = ( $files + 1 )

	# retrieve the unit cell
	cat $arg |\
	nawk '$1 == "CELL"{print $3,$4,$5,$6,$7,$8;exit}' |\
	cat >! ${tempfile}cell$files
	
	# retrieve the space group?
	
	# retrieve the sites (in fractional coordinates)
	cat $arg |\
	nawk '/^PLAN/{++plan} NF>=7 && plan {\
	    print $3,$4,$5, 1, 80, "frac"}' |\
	cat >! ${tempfile}xyz$files
	
	# remember the filenames
	if("$file1" == "") then
	    set file1 = "$arg"
	else
	    set file2 = "$arg"
	endif
	continue
    endif
    
    # recognize and convert SOLVE's solve.status output file
    if("$arg" =~ *solve.status) then
	if(! -e "$arg") then
	    echo "WARNING: $arg does not exist! "
	    continue
	endif
	@ files = ( $files + 1 )

	# retrieve the unit cell?
	set dir = `dirname $arg`
	foreach file ( `basename $arg` `ls -l $dir | sort -n +4 | nawk '/^-/ && mass<5000000 && len<900{print $NF; mass+=$5; len +=length($NF)+1}'` )
	    grep -l CELL ${dir}/$file >& /dev/null
	    # >& /dev/null
	    if(! $status) then
		nawk '$1 == "CELL"{print $2,$3,$4,$5,$6,$7;exit}' ${dir}/$file |\
		nawk '! /[^0-9 \.]/ && NF' >! ${tempfile}cell$files
		
		# make sure we got it
		set temp = `cat ${tempfile}cell$files`
		if($#temp == 6) then
		    echo "retrieving SOLVE cell from: ${dir}/$file"
		    break
		endif
		# keep looking
		rm -f ${tempfile}cell$files >& /dev/null
	    endif
	end
	
	# retrieve the sites from this SOLVE output file
	cat $arg |\
	nawk '/TOP SOLUTION FOUND BY SOLVE/{\
	while($1 != "TIME")\
	{getline;if((NF!=0)&&($1 != "TIME")) print}}' |\
	nawk '/OCC/{for(i in site){site[i]=""}; n=0}\
	    /[0-9]/{++n; site[n]= $0}\
	        END{for(i=1;i<=n;++i){print site[i]}}' |\
	nawk 'NF>5{print $(NF-5), $(NF-4), $(NF-3), $(NF-2), $(NF-1), "frac"}' |\
	cat >! ${tempfile}xyz$files
	
	# remember the filenames
	if("$file1" == "") then
	    set file1 = "$arg"
	else
	    set file2 = "$arg"
	endif
	continue
    endif
    
    # recognize and read mlphare scripts
    if("$arg" =~ *mlphare*) then
	if(! -e "$arg") then
	    echo "WARNING: $arg does not exist! "
	    continue
	endif
	grep "ATOM" "$arg" >& /dev/null
	if($status) then
	    echo "WARNING: $arg is not an mlphare script! "
	    continue
	endif

	@ files = ( $files + 1 )
	
	# try to roll-up cell from mtz file?
	echo "" >! ${tempfile}cell$files
	
	# what do we do with this?
	cat "$arg" |\
	nawk '$1~/^ATOM/{occ=$6; if(occ<$7) occ=$7;\
	      print $3, $4, $5, occ, $NF, "frac"}' |\
	cat >! ${tempfile}xyz$files

	# remember the filenames
	if("$file1" == "") then
	    set file1 = "$arg"
	else
	    set file2 = "$arg"
	endif
	continue
    endif
    
    if(-e "$arg") then
	# random file, we don't recognize the format
	@ files = ( $files + 1 )
	
	# see if we can get a cell ("smart" cell reader)
	cat $arg |\
	nawk 'toupper($1) == "CELL" && NF>=7{n=0\
	    for(i=1;i<=NF;++i){\
	      if($i+0>5 && $i+0==$i){cell = cell " " $i;++n};\
	      if(n==6){print cell; exit}}}' |\
	cat >! ${tempfile}cell$files
	
	# now see if we can read the coordinates
	# (look for last block of lines containing 3 consecutive numbers w 3 decimal places)
	cat $arg |\
	nawk 'toupper($1) != "CELL" && NF>=3{n=0;xyz="";\
	    for(i=1;i<=NF;++i){\
	      if($i ~ /[0-9]\.[0-9][0-9][0-9]/){xyz = xyz " " $i;++n};\
	      if(n==3){++sites; XYZ[sites]=xyz; next}}} \
	    sites && n<=3{Sites=sites;sites=0} \
	    END{if(sites) Sites=sites; \
	        for(i=1;i<=Sites;++i) print XYZ[i]}' |\
	cat >! ${tempfile}xyz
	
	set temp = `cat ${tempfile}xyz | wc -l`
	if($temp == 0) then
	    echo "WARNING: no sites found in $arg"
	    rm -f ${tempfile}xyz >& /dev/null
	    rm -f ${tempfile}cell$files >& /dev/null
	    @ files = ( $files - 1 )
	endif
	
	# is this fractional? or Angstrom?  (decide from range of values)
	set temp = `nawk 'BEGIN{RS=" "} {print}' ${tempfile}xyz | sort -n | nawk 'NF==1' | nawk '$1<-1.1 || $1 > 1.1' | wc -l`
	if($temp) then
	    nawk '{print $0, 1, 80, "orth"}' ${tempfile}xyz|\
	    cat >! ${tempfile}xyz$files
	else
	    nawk '{print $0, 1, 80, "frac"}' ${tempfile}xyz|\
	    cat >! ${tempfile}xyz$files
	endif
	rm -f ${tempfile}xyz >& /dev/null

	# remember the filenames
	if("$file1" == "") then
	    set file1 = "$arg"
	else
	    set file2 = "$arg"
	endif
	continue
    endif
end

goto ReturnFromSetup


exit
#########################################################################
# TODO:
#
#
#
#
#

    
resample:
#########################################################################
mapmask mapin $file2 mapout ${tempfile}extended.map << EOF >> $logfile
xyzlim 0 1 0 1 0 1
axis $AXIS2
EOF

# back-transform the second map to HKLs
sfall mapin ${tempfile}extended.map \
      hklout ${tempfile}map2.mtz << EOF | tee ${tempfile}.log >> $logfile
symm $SG2
mode sfcalc mapin
RESOLUTION $hiRES2
EOF
if($status) then
    # check axis order
    set temp = `cat ${tempfile}.log | nawk '/Check Iuvw/{printf "%c %c %c", 87+$6, 87+$7, 87+$8}' | nawk '! /[^ XYZ]/'`
    if(("$temp" != "$AXIS2")&&($#temp == 3)) then
	set AXIS2 = ( $temp )
	goto resample
    endif
    echo "ERROR: unsolvable problem with SFALL."
    echo "       perhaps it would be easier to just"
    echo "       re-calculate $map2 from wherever it came from"
    echo "       using: GRID $GRID1"
    echo "       in fft"
    exit 2
endif
rm -f ${tempfile}.log >& /dev/null

# no re-make the 2nd map on the 1st map's grid
fft hklin ${tempfile}map2.mtz mapout ${tempfile}.map << EOF >> $logfile
RESOLUTION 1000 $hiRES1
GRID $GRID1
LABIN F1=FC PHI=PHIC
EOF
rm -f ${tempfile}map2.mtz >& /dev/null

reorganize:
# and extend it to the same limits
mapmask mapin ${tempfile}.map mapout ${tempfile}map2.map << EOF >> $logfile
xyzlim $LIMITS1
axis   $AXIS1
pad    0
EOF
rm -f ${tempfile}.map >& /dev/null

set map2 = ${tempfile}map2.map  

goto resampled

